searchData={"items":[{"type":"module","doc":"This module gives an implementation for creating church numerals and the terms needed for addition and multiplication.","title":"HOL.ChurchNumerals","ref":"HOL.ChurchNumerals.html"},{"type":"module","doc":"iex> mk_num(1) |> PrettyPrint.pp_term(true)\n    \"(2 1. 2 1)\"\n\n    iex> plus(mk_num(1), mk_num(2)) |> PrettyPrint.pp_term(true)\n    \"(2 1. 2 (2 (2 1)))\"\n\n    iex> mult(mk_num(2), mk_num(3)) |> PrettyPrint.pp_term(true)\n    \"(2 1. 2 (2 (2 (2 (2 (2 1))))))\"\n\n    iex> mult_term() |> PrettyPrint.pp_term(true)\n    \"(6 5 4 3. 6 (2. 5 (1. 4 1) 2) 3)\"","title":"Examples - HOL.ChurchNumerals","ref":"HOL.ChurchNumerals.html#module-examples"},{"type":"function","doc":"Returns a lambda term representing the given integer as a church numeral","title":"HOL.ChurchNumerals.mk_num/1","ref":"HOL.ChurchNumerals.html#mk_num/1"},{"type":"function","doc":"Applies the `mult_term()` to the given numbers.\n\nShorthand for `mk_appl_term(mk_appl_term(mult_term(), m), n)`","title":"HOL.ChurchNumerals.mult/2","ref":"HOL.ChurchNumerals.html#mult/2"},{"type":"function","doc":"Returns the lamda term that represents the multiplication function: `λ 6 5 4 3. 6 (2. 5 (1. 4 1) 2) 3`","title":"HOL.ChurchNumerals.mult_term/0","ref":"HOL.ChurchNumerals.html#mult_term/0"},{"type":"function","doc":"Applies the `plus_term()` to the given numbers.\n\nShorthand for `mk_appl_term(mk_appl_term(plus_term(), m), n)`","title":"HOL.ChurchNumerals.plus/2","ref":"HOL.ChurchNumerals.html#plus/2"},{"type":"function","doc":"Returns the lamda term that represents the addition function: \"λ 5 4 3 2. 5 (1. 3 1) (4 (1. 3 1) 2\"","title":"HOL.ChurchNumerals.plus_term/0","ref":"HOL.ChurchNumerals.html#plus_term/0"},{"type":"function","doc":"Applies the `succ_term()` to a given number.\n\nShorthand for `mk_appl_term(succ_term(), n)`","title":"HOL.ChurchNumerals.succ/1","ref":"HOL.ChurchNumerals.html#succ/1"},{"type":"function","doc":"Returns the lamda term that represents the successor function: \"λ 4 3 2. 3 (4 (1. 3 1) 2)\"","title":"HOL.ChurchNumerals.succ_term/0","ref":"HOL.ChurchNumerals.html#succ_term/0"},{"type":"module","doc":"This module defines the datastructures used in the application","title":"HOL.Data","ref":"HOL.Data.html"},{"type":"type","doc":"Represents a bound variable.\n\nThe name must be a positive integer.","title":"HOL.Data.bound_var_decl/0","ref":"HOL.Data.html#t:bound_var_decl/0"},{"type":"type","doc":"Represents a constant.\n\nThe name must be a string.","title":"HOL.Data.const_decl/0","ref":"HOL.Data.html#t:const_decl/0"},{"type":"type","doc":"Combination type representing free variables, bound variables and constants.\n\n** Key ** | **Data stored** | **Accessor Function**\n:kind | Whether it is a free or bound variable or constant | `get_kind/1`\n:name | Name | `get_name/1`\n:type | Type | `get_type/1`","title":"HOL.Data.declaration/0","ref":"HOL.Data.html#t:declaration/0"},{"type":"macro","doc":"Creates a new declaration. This function should not need be called under normal usage of this Package\n\nThis Dataformat uses a `Record`.","title":"HOL.Data.declaration/1","ref":"HOL.Data.html#declaration/1"},{"type":"macro","doc":"","title":"HOL.Data.declaration/2","ref":"HOL.Data.html#declaration/2"},{"type":"type","doc":"Represents a free variable.\n\nThe name can be either a string or a reference.","title":"HOL.Data.free_var_decl/0","ref":"HOL.Data.html#t:free_var_decl/0"},{"type":"function","doc":"Accessor function for argument types","title":"HOL.Data.get_arg_types/1","ref":"HOL.Data.html#get_arg_types/1"},{"type":"function","doc":"Accessor function for the arguments of the term","title":"HOL.Data.get_args/1","ref":"HOL.Data.html#get_args/1"},{"type":"function","doc":"Accessor function for the bound variables of the term","title":"HOL.Data.get_bvars/1","ref":"HOL.Data.html#get_bvars/1"},{"type":"function","doc":"Accessor function for the free variable of the substitution","title":"HOL.Data.get_fvar/1","ref":"HOL.Data.html#get_fvar/1"},{"type":"function","doc":"Accessor function for the free variables in the term","title":"HOL.Data.get_fvars/1","ref":"HOL.Data.html#get_fvars/1"},{"type":"function","doc":"Accessor function for goal type","title":"HOL.Data.get_goal_type/1","ref":"HOL.Data.html#get_goal_type/1"},{"type":"function","doc":"Accessor function for head of the term","title":"HOL.Data.get_head/1","ref":"HOL.Data.html#get_head/1"},{"type":"function","doc":"Accessor function for type of declaration.\n\nReturns\n  - `:fv` when given a free variable\n  - `:bv` when given a bound variable\n  - `:co` when given a constant","title":"HOL.Data.get_kind/1","ref":"HOL.Data.html#get_kind/1"},{"type":"function","doc":"Accessor function for the max_num of the term","title":"HOL.Data.get_max_num/1","ref":"HOL.Data.html#get_max_num/1"},{"type":"function","doc":"Accessor function for the identifier","title":"HOL.Data.get_name/1","ref":"HOL.Data.html#get_name/1"},{"type":"function","doc":"Accessor function for the term of the substitution","title":"HOL.Data.get_term/1","ref":"HOL.Data.html#get_term/1"},{"type":"function","doc":"Accessor function for the type of the term","title":"HOL.Data.get_term_type/1","ref":"HOL.Data.html#get_term_type/1"},{"type":"function","doc":"Accessor function for the type","title":"HOL.Data.get_type/1","ref":"HOL.Data.html#get_type/1"},{"type":"type","doc":"Represents a term in the simply typed lambda calculus.\n\nTerms are always automatically beta-reduced and eta-expanded.\n\n** Key ** | **Data stored** | **Accessor Function**\n:bvars | A list of bound variables that are bound here | `get_bvars/1`\n:head | The head of the term | `get_head/1`\n:args | A list of terms that are applied to the head | `get_args/1`\n:type | The type of the term | `get_term_type/1`\n:fvars | A list of all free variables in the term | `get_fvars/1`\n:max_num | The highest bound variable that is bound here or in one of the args | `get_max_num/1`","title":"HOL.Data.hol_term/0","ref":"HOL.Data.html#t:hol_term/0"},{"type":"macro","doc":"Creates a new term.\n\nFor creating new terms see `HOL.Terms.mk_term/1`\n\nThis function should not need be called under normal usage of this Package.\n\nThis Dataformat uses a `Record`.","title":"HOL.Data.hol_term/1","ref":"HOL.Data.html#hol_term/1"},{"type":"macro","doc":"","title":"HOL.Data.hol_term/2","ref":"HOL.Data.html#hol_term/2"},{"type":"function","doc":"Creates a bound variable with the given id and type.\n\nThis function should not need be called under normal usage of this Package","title":"HOL.Data.mk_bound_var/2","ref":"HOL.Data.html#mk_bound_var/2"},{"type":"function","doc":"iex> mk_bound_var(1, mk_type(:i))\n    {:decl, :bv, 1, {:type, :i, []}}","title":"Example - HOL.Data.mk_bound_var/2","ref":"HOL.Data.html#mk_bound_var/2-example"},{"type":"function","doc":"Creates a constant with the given name and type.","title":"HOL.Data.mk_const/2","ref":"HOL.Data.html#mk_const/2"},{"type":"function","doc":"iex> mk_const(\"x\", mk_type(:i))\n    {:decl, :co, \"x\", {:type, :i, []}}","title":"Example - HOL.Data.mk_const/2","ref":"HOL.Data.html#mk_const/2-example"},{"type":"function","doc":"Creates a free variable with the given name and type.\n\nAlso see function `mk_uniqe_var/1` for creating a uniqe variable","title":"HOL.Data.mk_free_var/2","ref":"HOL.Data.html#mk_free_var/2"},{"type":"function","doc":"iex> mk_free_var(\"x\", mk_type(:i))\n    {:decl, :fv, \"x\", {:type, :i, []}}","title":"Example - HOL.Data.mk_free_var/2","ref":"HOL.Data.html#mk_free_var/2-example"},{"type":"function","doc":"Creates a new substitution","title":"HOL.Data.mk_substitution/2","ref":"HOL.Data.html#mk_substitution/2"},{"type":"function","doc":"Creates a base type without arguments.","title":"HOL.Data.mk_type/1","ref":"HOL.Data.html#mk_type/1"},{"type":"function","doc":"# Creates base type i\n    iex> mk_type(:i)\n    {:type, :i, []}\n\n    # Creates base type o\n    iex> mk_type(:o)\n    {:type, :o, []}","title":"Examples - HOL.Data.mk_type/1","ref":"HOL.Data.html#mk_type/1-examples"},{"type":"function","doc":"Creates a type with the given arguments.\n\nThe goal type is always stored as an atom, but it doesn't have to be given as such","title":"HOL.Data.mk_type/2","ref":"HOL.Data.html#mk_type/2"},{"type":"function","doc":"- goal_type: Atom or `type()` that represents the goal type\n  - arg_types: List of `type()` that represents the argument types","title":"Parameters - HOL.Data.mk_type/2","ref":"HOL.Data.html#mk_type/2-parameters"},{"type":"function","doc":"# Creates type i->i\n    iex> mk_type(:i, [mk_type(:i)])\n    {:type, :i, [{:type, :i, []}]}\n\n    # Creates type i->i->i\n    iex> mk_type(mk_type(:o, [mk_type(:i)]), [mk_type(:i)])\n    {:type, :o, [{:type, :i, []}, {:type, :i, []}]}\n\n    # Creates type i->i->i (identical to previous)\n    iex> mk_type(:o, [mk_type(:i), mk_type(:i)])\n    {:type, :o, [{:type, :i, []}, {:type, :i, []}]}\n\n    # Creates type (i->i)->i->i\n    iex> mk_type(:i, [mk_type(:i, [mk_type(:i)]), mk_type(:i)])\n    {:type, :i, [{:type, :i, [{:type, :i, []}]}, {:type, :i, []}]}","title":"Examples - HOL.Data.mk_type/2","ref":"HOL.Data.html#mk_type/2-examples"},{"type":"function","doc":"Creates a free variable with a unique name and type.\n\nThe name is created via the function `Kernel.make_ref/0`. See the corresponding documentation for the limitations of this function.\n\nAlso see function `mk_free_var/2` for creating a named free variable","title":"HOL.Data.mk_uniqe_var/1","ref":"HOL.Data.html#mk_uniqe_var/1"},{"type":"type","doc":"Represents a substitution of a free variable with a term\n\n**Key** | **Data stored** | **Accessor Function**\n:fvar | Free Variable to Replace | `get_fvar/1`\n:term | Replacement Term | `get_term/1`","title":"HOL.Data.substitution/0","ref":"HOL.Data.html#t:substitution/0"},{"type":"macro","doc":"Creates a new substitution.\n\nThis Dataformat uses a `Record`.","title":"HOL.Data.substitution/1","ref":"HOL.Data.html#substitution/1"},{"type":"macro","doc":"","title":"HOL.Data.substitution/2","ref":"HOL.Data.html#substitution/2"},{"type":"type","doc":"This type encodes the type information for a HOL term or declaration.\n\nIt has its goal type stored as an atom, with all input types in a list.\n\n** Key ** | **Data stored** | **Accessor Function**\n:goal | Goal Type | `get_goal_type/1`\n:args | List of argument types | `get_arg_types/1`","title":"HOL.Data.type/0","ref":"HOL.Data.html#t:type/0"},{"type":"type","doc":"# base type i is encoded as\n      iex> mk_type(:i)\n      {:type, :i, []}\n\n      # function type i->o is encoded as\n      iex> mk_type(:o, [mk_type(:i)])\n      {:type, :o, [{:type, :i, []}]}","title":"Examples - HOL.Data.type/0","ref":"HOL.Data.html#t:type/0-examples"},{"type":"macro","doc":"Creates a new type. This function should not need be called under normal usage of this Package\n\nThis Dataformat uses a `Record`.","title":"HOL.Data.type/1","ref":"HOL.Data.html#type/1"},{"type":"macro","doc":"","title":"HOL.Data.type/2","ref":"HOL.Data.html#type/2"},{"type":"module","doc":"This module defines the datastructures used in the unification","title":"HOL.Data.Unification","ref":"HOL.Data.Unification.html"},{"type":"function","doc":"Accessor function for the flexlist","title":"HOL.Data.Unification.get_flexlist/1","ref":"HOL.Data.Unification.html#get_flexlist/1"},{"type":"function","doc":"Accessor function for the max_depth count","title":"HOL.Data.Unification.get_max_depth_count/1","ref":"HOL.Data.Unification.html#get_max_depth_count/1"},{"type":"function","doc":"Accessor function for the solutions","title":"HOL.Data.Unification.get_solutions/1","ref":"HOL.Data.Unification.html#get_solutions/1"},{"type":"function","doc":"Accessor function for the substitutions","title":"HOL.Data.Unification.get_substitutions/1","ref":"HOL.Data.Unification.html#get_substitutions/1"},{"type":"type","doc":"Contains two terms in a tuple.","title":"HOL.Data.Unification.term_pair/0","ref":"HOL.Data.Unification.html#t:term_pair/0"},{"type":"type","doc":"Represents the summarized results from the unification.\n\n**Key** | **Data stored** | **Accessor Function**\n:solutions | List of unification_solutions | `get_solutions/1`\n:max_depth_reached_count | The amount of branches that couldn't terminate due to the max_depth limit | `get_max_depth_count/1`","title":"HOL.Data.Unification.unification_results/0","ref":"HOL.Data.Unification.html#t:unification_results/0"},{"type":"macro","doc":"Creates a new Unification Result.\n\nThis function should not need be called under normal usage of this Package.\n\nThis Dataformat uses a `Record`.","title":"HOL.Data.Unification.unification_results/1","ref":"HOL.Data.Unification.html#unification_results/1"},{"type":"macro","doc":"","title":"HOL.Data.Unification.unification_results/2","ref":"HOL.Data.Unification.html#unification_results/2"},{"type":"type","doc":"Represents a single solution of a unification problem.\n\nThe flexlist represents cases where the head of two terms are both free variables.\nAs this situation has many (often infinite) solutions, no concrete substitutions are created.\n\n**Key** | **Data stored** | **Accessor Function**\n:substitutions | The substitutions needed to unify the terms | `get_substitutions/1`\n:flexlist | List of flexflex pairs | `get_flexlist/1`","title":"HOL.Data.Unification.unification_solution/0","ref":"HOL.Data.Unification.html#t:unification_solution/0"},{"type":"macro","doc":"Creates a new Unification Solution.\n\nThis function should not need be called under normal usage of this Package.\n\nThis Dataformat uses a `Record`.","title":"HOL.Data.Unification.unification_solution/1","ref":"HOL.Data.Unification.html#unification_solution/1"},{"type":"macro","doc":"","title":"HOL.Data.Unification.unification_solution/2","ref":"HOL.Data.Unification.html#unification_solution/2"},{"type":"module","doc":"This module implements the substitution.\nIt can be used to replace free variables in a term with any other terms.","title":"HOL.Substitution","ref":"HOL.Substitution.html"},{"type":"module","doc":"# Create Term\n    iex> x = mk_free_var(\"x\", mk_type(:i, [mk_type(:i)]))\n    iex> term = mk_term(x) |> mk_appl_term(mk_const_term(\"c\", mk_type(:i)))\n    iex> PrettyPrint.pp_term(term)\n    \"(x c)\"\n    # Create substitution\n    iex> term_y = mk_free_var_term(\"y\", mk_type(:i, [mk_type(:i)]))\n    iex> substitution = mk_substitution(x, term_y)\n    iex> subst(substitution, term) |> PrettyPrint.pp_term()\n    \"(y c)\"","title":"Example - HOL.Substitution","ref":"HOL.Substitution.html#module-example"},{"type":"function","doc":"Adds a single substitution to a list of substitutions.\nThe new substitution is applied to all terms in the other substitutions, keeping the list idempotent.\n\nSubstitutions using free variables with references as names are not added to the list,\nbut still apply their substitution to the terms.","title":"HOL.Substitution.add_subst/2","ref":"HOL.Substitution.html#add_subst/2"},{"type":"function","doc":"iex> x = mk_free_var(\"x\", mk_type(:i))\n    iex> y = mk_free_var(\"y\", mk_type(:i))\n    iex> subst_list = [mk_substitution(y, mk_term(x))]\n    iex> new_subst = mk_substitution(x, mk_const_term(\"c\", mk_type(:i)))\n    iex> add_subst(subst_list, new_subst) |> PrettyPrint.pp_subst()\n    \"x <- c | y <- c\"","title":"Example - HOL.Substitution.add_subst/2","ref":"HOL.Substitution.html#add_subst/2-example"},{"type":"function","doc":"Applies a singular substitution or a list of substitutions to a term","title":"HOL.Substitution.subst/2","ref":"HOL.Substitution.html#subst/2"},{"type":"function","doc":"# Create Term\n    iex> x = mk_free_var(\"x\", mk_type(:i, [mk_type(:i)]))\n    iex> term = mk_term(x) |> mk_appl_term(mk_const_term(\"c\", mk_type(:i)))\n    iex> PrettyPrint.pp_term(term)\n    \"(x c)\"\n    # Create substitution\n    iex> term_y = mk_free_var_term(\"y\", mk_type(:i, [mk_type(:i)]))\n    iex> substitution = mk_substitution(x, term_y)\n    iex> subst(substitution, term) |> PrettyPrint.pp_term()\n    \"(y c)\"","title":"Example - HOL.Substitution.subst/2","ref":"HOL.Substitution.html#subst/2-example"},{"type":"module","doc":"This module gives functions for creating terms, applications and abstractions.","title":"HOL.Terms","ref":"HOL.Terms.html"},{"type":"module","doc":"iex> var_x = HOL.Data.mk_free_var(\"x\", mk_type(:i))\n    iex> const_f = HOL.Data.mk_const(\"f\", mk_type(:i, [mk_type(:i)]))\n    iex> application = mk_appl_term(mk_term(const_f), mk_term(var_x))\n    iex> PrettyPrint.pp_term(application)\n    \"(f x)\"\n    iex> abstraction = mk_abstr_term(application, var_x)\n    iex> PrettyPrint.pp_term(abstraction)\n    \"(1. f 1)\"\n    iex> mk_appl_term(abstraction, mk_const_term(\"c\", mk_type(:i))) |> PrettyPrint.pp_term()\n    \"(f c)\"","title":"Examples - HOL.Terms","ref":"HOL.Terms.html#module-examples"},{"type":"function","doc":"Adjusts all bound variables so that the innermost terms have the lowest bound variables.\n\n> #### Warning {: .warning}\n>\n> This function can break if it is applied to terms with bound variables that are not bound anywhere!\n\nThis function is only needed when interacting with terms in unusual ways.\nUsing `mk_abstr_term/2` or `mk_appl_term/2` applies this function automatically.","title":"HOL.Terms.adjust_all_bound_vars/1","ref":"HOL.Terms.html#adjust_all_bound_vars/1"},{"type":"function","doc":"Adjusts the outer layer of bound variables so that those bound variables are as\nsmall as possible without overlapping with bound variables defined in the arguments\n\n> #### Warning {: .warning}\n>\n> This function can break if it is applied to terms with bound variables that are not bound anywhere!\n\nThis function is only needed when interacting with terms in unusual ways.\nUsing `mk_abstr_term/2` or `mk_appl_term/2` applies this function automatically.","title":"HOL.Terms.adjust_outer_bound_vars/1","ref":"HOL.Terms.html#adjust_outer_bound_vars/1"},{"type":"function","doc":"Abstracts a free variable from a term by replacing it with a bound variable.\n\nOnly free variables can be abstracted.","title":"HOL.Terms.mk_abstr_term/2","ref":"HOL.Terms.html#mk_abstr_term/2"},{"type":"function","doc":"- term: Term in which to abstract a variable\n  - var: Free Variable to abstract","title":"Parameters - HOL.Terms.mk_abstr_term/2","ref":"HOL.Terms.html#mk_abstr_term/2-parameters"},{"type":"function","doc":"iex> var_x = mk_free_var(\"x\", mk_type(:i))\n    iex> term_f = mk_const_term(\"f\", mk_type(:i, [mk_type(:i)]))\n    iex> application = mk_appl_term(term_f, mk_term(var_x))\n    iex> PrettyPrint.pp_term(application)\n    \"(f x)\"\n    iex> abstraction = mk_abstr_term(application, var_x)\n    iex> PrettyPrint.pp_term(abstraction)\n    \"(1. f 1)\"","title":"Examples - HOL.Terms.mk_abstr_term/2","ref":"HOL.Terms.html#mk_abstr_term/2-examples"},{"type":"function","doc":"Applies one term to another.\n\n# Example\n\n    iex> var_x = mk_free_var(\"x\", mk_type(:i))\n    iex> term_f = mk_const_term(\"f\", mk_type(:i, [mk_type(:i)]))\n    iex> application = mk_appl_term(term_f, mk_term(var_x))\n    iex> PrettyPrint.pp_term(application)\n    \"(f x)\"\n    iex> abstraction = mk_abstr_term(application, var_x)\n    iex> PrettyPrint.pp_term(abstraction)\n    \"(1. f 1)\"\n    iex> mk_appl_term(abstraction, mk_const_term(\"c\", mk_type(:i))) |> PrettyPrint.pp_term()\n    \"(f c)\"","title":"HOL.Terms.mk_appl_term/2","ref":"HOL.Terms.html#mk_appl_term/2"},{"type":"function","doc":"Creates a term with a free variable as head, that has the given name and type.","title":"HOL.Terms.mk_const_term/2","ref":"HOL.Terms.html#mk_const_term/2"},{"type":"function","doc":"Creates a term with a free variable as head, that has the given name and type.","title":"HOL.Terms.mk_free_var_term/2","ref":"HOL.Terms.html#mk_free_var_term/2"},{"type":"function","doc":"Creates a term from a declaration with this declaration as the head. Terms are always eta-expanded.\n\nAlso see `mk_free_var_term/2`and `mk_const_term/2` to create a term, without first creating a declaration.","title":"HOL.Terms.mk_term/1","ref":"HOL.Terms.html#mk_term/1"},{"type":"function","doc":"iex> mk_term(HOL.Data.mk_const(\"x\", mk_type(:i)))\n    {:term, [], {:decl, :co, \"x\", {:type, :i, []}}, [],  mk_type(:i), [], 0}","title":"Example - HOL.Terms.mk_term/1","ref":"HOL.Terms.html#mk_term/1-example"},{"type":"module","doc":"This module provides the function `unify/3` that tries to unify two terms.","title":"HOL.Unification","ref":"HOL.Unification.html"},{"type":"function","doc":"This function tries to unify a set of terms with each other.\n\nReturns a `HOL.Data.Unification.unification_results/0` object that contains the\nsummarized results.","title":"HOL.Unification.unify/3","ref":"HOL.Unification.html#unify/3"},{"type":"function","doc":"- terms: A tuple containing two terms or a list of such tuples.\n  The function only allows tuples with two terms.\n  - return_all_solutions: If true tries to find all possible solutions it can.\n  If false stops execution after finding one solution.\n  - max_depth: How deep the function should look for solutions. The amount of branches that\n  abort due to this constraint are reported in the result. If this value is set to a\n  negative integer this constraint is removed.\n\n> #### Warning {: .warning}\n>\n> This function is not guaranteed to terminate without a max_depth limit!","title":"Parameters - HOL.Unification.unify/3","ref":"HOL.Unification.html#unify/3-parameters"},{"type":"function","doc":"# Create two terms to unify\n    iex> type_i = mk_type(:i, [])\n    iex> type_ii = mk_type(:i, [type_i])\n    iex> term_a = mk_free_var_term(\"x\", type_ii) |> mk_appl_term(mk_const_term(\"c\", type_i))\n    iex> PrettyPrint.pp_term(term_a)\n    \"(x c)\"\n    iex> term_b = mk_const_term(\"f\", type_ii) |> mk_appl_term(mk_const_term(\"d\", type_i))\n    iex> PrettyPrint.pp_term(term_b)\n    \"(f d)\"\n    iex> input = {term_a, term_b}\n    iex> result = unify(input)\n    iex> PrettyPrint.pp_res(result)\n    \"{ max_depth reached 0 times; solutions: [{x <- (1. f d) , []}]}\"\n    iex> get_solutions(result) |> hd() |> PrettyPrint.pp_sol()\n    \"{x <- (1. f d) , []}\"","title":"Example - HOL.Unification.unify/3","ref":"HOL.Unification.html#unify/3-example"},{"type":"module","doc":"This module defines functions to determine the imitation and projection bindings neccessary\nfor unification.","title":"HOL.Unification.Bindings","ref":"HOL.Unification.Bindings.html"},{"type":"function","doc":"Gives the arity of a given type. The arity is the amount of argument types it has.","title":"HOL.Unification.Bindings.arity/1","ref":"HOL.Unification.Bindings.html#arity/1"},{"type":"type","doc":"The two binding types available","title":"HOL.Unification.Bindings.binding_type/0","ref":"HOL.Unification.Bindings.html#t:binding_type/0"},{"type":"function","doc":"Returns the bindings determined by the two declarations given.\n\nIt only creates the bindings that are requested in the binding_types list.","title":"HOL.Unification.Bindings.generic_binding/3","ref":"HOL.Unification.Bindings.html#generic_binding/3"},{"type":"module","doc":"Defines functions to turn various datatypes into strings","title":"PrettyPrint","ref":"PrettyPrint.html"},{"type":"function","doc":"","title":"PrettyPrint.pp_decl/2","ref":"PrettyPrint.html#pp_decl/2"},{"type":"function","doc":"","title":"PrettyPrint.pp_res/2","ref":"PrettyPrint.html#pp_res/2"},{"type":"function","doc":"","title":"PrettyPrint.pp_sol/2","ref":"PrettyPrint.html#pp_sol/2"},{"type":"function","doc":"","title":"PrettyPrint.pp_subst/2","ref":"PrettyPrint.html#pp_subst/2"},{"type":"function","doc":"","title":"PrettyPrint.pp_term/2","ref":"PrettyPrint.html#pp_term/2"},{"type":"function","doc":"","title":"PrettyPrint.pp_tls/2","ref":"PrettyPrint.html#pp_tls/2"},{"type":"function","doc":"","title":"PrettyPrint.pp_type/1","ref":"PrettyPrint.html#pp_type/1"},{"type":"module","doc":"This module currently only contains functions to express a unification problem as a\nTPTP-Problem (see `unification_problem_to_tptp`/1) and some helper functions.","title":"TPTPParser","ref":"TPTPParser.html"},{"type":"function","doc":"Outputs a variable name as a string that can be used inside of TPTP Problems.\n\nBound Variables have a `BV_` prepended before the number.\n\nConstant names are unchanged.\n\nFree Variables are forced into uppercase.\n\n> #### Warning {: .warning}\n>\n> Variable names are generally case sensitive (\"x\" != \"X\"). This is not possible in TPTP and may cause issues.","title":"TPTPParser.get_var_name/1","ref":"TPTPParser.html#get_var_name/1"},{"type":"function","doc":"Transform a term into a TPTP parsable term string","title":"TPTPParser.term_to_tptp/1","ref":"TPTPParser.html#term_to_tptp/1"},{"type":"function","doc":"Transform a type into a TPTP parsable type string","title":"TPTPParser.type_to_tptp/2","ref":"TPTPParser.html#type_to_tptp/2"},{"type":"function","doc":"Transform an unification problem into a TPTP parsable problem","title":"TPTPParser.unification_problem_to_tptp/1","ref":"TPTPParser.html#unification_problem_to_tptp/1"},{"type":"function","doc":"Transform a variable into a TPTP parsable variable string","title":"TPTPParser.vars_to_tptp_declaration/1","ref":"TPTPParser.html#vars_to_tptp_declaration/1"},{"type":"extras","doc":"# HOL\n\nThis elixir package gives an implementation of higher order logic via the [simply typed lambda calculus](https://en.wikipedia.org/wiki/Simply_typed_lambda_calculus) and higher order pre-[unification](https://en.wikipedia.org/wiki/Unification_%28computer_science%29#Higher-order_unification). All lambda terms are always automatically beta-reduced and eta-expanded. Bound Variables are automatically named via [de bruijn indices](https://en.wikipedia.org/wiki/De_Bruijn_index).\n\nThis package was developed at [University of Bamberg](https://www.uni-bamberg.de/en/) with the [AISE Chair](https://www.uni-bamberg.de/en/aise/).","title":"HOL","ref":"readme.html"},{"type":"extras","doc":"This package can be installed by adding `hol` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hol, \"1.0.1\"}\n  ]\nend\n```","title":"Installation - HOL","ref":"readme.html#installation"},{"type":"extras","doc":"Documentation is available on [hexdocs](https://hexdocs.pm/hol/readme.html)","title":"Documentation - HOL","ref":"readme.html#documentation"},{"type":"extras","doc":"- `HOL.Data.mk_type/2`: To create a type for the simply typed lambda calculus\n- `HOL.Data.mk_free_var/2`: To create a free variable\n- `HOL.Data.mk_const/2`: To create a constant\n- `HOL.Terms.mk_term/1`: To create a term from a variable or constant\n- `HOL.Terms.mk_appl_term/2`: To apply to terms to each other\n- `HOL.Terms.mk_abstr_term/2`: To create a lambda abstraction\n- `HOL.Data.mk_substitution/2`: To create a substitution\n- `HOL.Substitution.subst/2`: To apply a list of substitutions to a term\n- `HOL.Unification.unify/3`: To find substitutions to unify two terms\n- Use the functions in the `PrettyPrint` Module to show data in a humanly readable format","title":"Most important functions - HOL","ref":"readme.html#most-important-functions"},{"type":"extras","doc":"The following and more examples can be found interactively in the [livebook](https://hexdocs.pm/hol/unification_examples.html).\n\nSetup for Tests:\n\n```elixir\ntype_i = mk_type(:i)\ntype_ii = mk_type(type_i, [type_i])\ntype_ii_ii = mk_type(type_ii, [type_ii])\ntype_iii = mk_type(type_i, [type_i, type_i])\n```","title":"Unification Examples - HOL","ref":"readme.html#unification-examples"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_ii_ii)\n\nx_times_10 = mult_term() |> mk_appl_term(t_x) |> mk_appl_term(mk_num(10))\n\ninput = [\n  {x_times_10, mk_num(1000)}\n]\n\nresult = unify(input, true, 1000)\npp_res(result, true)\n```\n\nThis finds one substitution that replaces `x` with the church numeral `100`\n\nx = 100: `x <- (2 1. 2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 1) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) )`","title":"Example: x \\* 10 = 1000 - HOL","ref":"readme.html#example-x-10-1000"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_iii)\nt_a = mk_const_term(\"a\", type_i)\nt_f = mk_const_term(\"f\", type_iii)\n\nxaa = mk_appl_term(mk_appl_term(t_x, t_a), t_a)\nfaa = mk_appl_term(mk_appl_term(t_f, t_a), t_a)\n\ninput = {xaa, faa}\n\nresult = unify(input)\npp_res(result)\n```\n\nFinds nine possible substitutions for unification:\n\n`x <- λ 2 1. f 1 1`\n\n`x <- λ 2 1. f 1 2`\n\n`x <- λ 2 1. f 1 a`\n\n`x <- λ 2 1. f 2 1`\n\n`x <- λ 2 1. f 2 2`\n\n`x <- λ 2 1. f 2 a`\n\n`x <- λ 2 1. f a 1`\n\n`x <- λ 2 1. f a 2`\n\n`x <- λ 2 1. f a a`","title":"Example: xaa = faa - HOL","ref":"readme.html#example-xaa-faa"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_ii)\nt_f = mk_const_term(\"f\", type_ii)\nt_a = mk_const_term(\"a\", type_i)\n\nxfa = mk_appl_term(t_x, mk_appl_term(t_f, t_a))\nfxa = mk_appl_term(t_f, mk_appl_term(t_x, t_a))\n\ndepth = 5\ninput = {xfa, fxa}\n\nresult = unify(input, true, depth)\npp_res(result)\n```\n\nThis finds four solutions:\n\n`x <- λ 1. 1 `\n\n`x <- λ 1. f 1`\n\n`x <- λ 1. f (f 1)`\n\n`x <- λ 1. f (f (f 1))`\n\nHowever the maximum search depth was also reached twice. This means that it might not have found all solutions. Try changing the `depth` value and see what other solutions can be found!","title":"Example: x(fa) = f(xa) - HOL","ref":"readme.html#example-x-fa-f-xa"},{"type":"extras","doc":"- xy+z=21\n- x+y+z=10\n- xz+y=9\n\n```elixir\nt_x = mk_free_var_term(\"x\", type_ii_ii)\nt_y = mk_free_var_term(\"y\", type_ii_ii)\nt_z = mk_free_var_term(\"z\", type_ii_ii)\n\ninput = [\n  {plus(mult(t_x, t_y), t_z), mk_num(21)},\n  {plus(plus(t_x, t_y), t_z), mk_num(10)},\n  {plus(mult(t_x, t_z), t_y), mk_num(9)}\n]\n\ndepth = 50\nresult = unify(input, true, depth)\npp_res(result, true)\n```\n\nTwo Solutions are found here:\n\nz = 1, y = 4, x = 5: `z <- λ 2 1. 2 1` | `y <- λ 2 1. 2 (2 (2 (2 1)))` | `x <- λ 2 1. 2 (2 (2 (2 (2 1))))`\n\nz = 1, y = 5, x = 4: `z <- λ 2 1. 2 1` | `y <- λ 2 1. 2 (2 (2 (2 (2 1))))` | `x <- λ 2 1. 2 (2 (2 (2 1)))`\n\nOnce again the maximum search depth was reached multiple times. However increasing the search depth will not yield more solutions here. Unification doesn't always terminate even when no more solutions can be found!","title":"Example: Multiple Equations - HOL","ref":"readme.html#example-multiple-equations"},{"type":"extras","doc":"# Unification Examples\n\n```elixir\nMix.install([{:hol, \"1.0.1\"}])\n```","title":"Unification Examples","ref":"unification_examples.html"},{"type":"extras","doc":"```elixir\nimport HOL.Data\nimport HOL.Terms\nimport HOL.ChurchNumerals\nimport HOL.Unification\nimport PrettyPrint\n\nLoggerHelper.set_logger_warning()\n```\n\nHere we setup the types we need to construct the following examples\n\n```elixir\ntype_i = mk_type(:i)\ntype_ii = mk_type(type_i, [type_i])\ntype_ii_ii = mk_type(type_ii, [type_ii])\ntype_iii = mk_type(type_i, [type_i, type_i])\n```","title":"Setup - Unification Examples","ref":"unification_examples.html#setup"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_ii_ii)\n\nx_times_5 = mult_term() |> mk_appl_term(t_x) |> mk_appl_term(mk_num(5))\ninput = {x_times_5, mk_num(30)}\n\nresult = unify(input)\npp_res(result, true)\n```\n\nThis finds one substitution that replaces `x` with the church numeral `6`\n\nx = 6: `x <- λ 2 1. 2 (2 (2 (2 (2 (2 1)))))`","title":"Example: x \\* 5 = 30 - Unification Examples","ref":"unification_examples.html#example-x-5-30"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_ii_ii)\n\nx_times_10 = mult_term() |> mk_appl_term(t_x) |> mk_appl_term(mk_num(10))\n\ninput = [\n  {x_times_10, mk_num(1000)}\n]\n\nresult = unify(input, true, 1000)\npp_res(result, true)\n```\n\nThis finds one substitution that replaces `x` with the church numeral `100`\n\nx = 100: `x <- (2 1. 2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 (2 1) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) ) )`","title":"Example: x \\* 10 = 1000 - Unification Examples","ref":"unification_examples.html#example-x-10-1000"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_ii_ii)\nt_y = mk_free_var_term(\"y\", type_ii_ii)\n\nx_times_y = mult_term() |> mk_appl_term(t_x) |> mk_appl_term(t_y)\ninput = {x_times_y, mk_num(4)}\n\nresult = unify(input, true, 40)\npp_res(result, true)\n\n# 1*4=4 & 4*1=4 & 2*2=4\n```\n\nThis finds three solutions:\n\ny = 1, x = 4: `y <- λ 2 1. 2 1` | `x <- λ 2 1. 2 (2 (2 (2 1)))`\n\ny = 2, x = 2: `y <- λ 2 1. 2 (2 1)` | `x <- λ 2 1. 2 (2 1)`\n\ny = 4, x = 1: `y <- λ 2 1. 2 (2 (2 (2 1)))` | `x <- λ 2 1. 2 1`","title":"Example: x \\* y = 4 - Unification Examples","ref":"unification_examples.html#example-x-y-4"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_iii)\nt_a = mk_const_term(\"a\", type_i)\nt_f = mk_const_term(\"f\", type_iii)\n\nxaa = mk_appl_term(mk_appl_term(t_x, t_a), t_a)\nfaa = mk_appl_term(mk_appl_term(t_f, t_a), t_a)\n\ninput = {xaa, faa}\n\nresult = unify(input)\npp_res(result)\n```\n\nFinds nine possible substitutions for unification:\n\n`x <- λ 2 1. f 1 1`\n\n`x <- λ 2 1. f 1 2`\n\n`x <- λ 2 1. f 1 a`\n\n`x <- λ 2 1. f 2 1`\n\n`x <- λ 2 1. f 2 2`\n\n`x <- λ 2 1. f 2 a`\n\n`x <- λ 2 1. f a 1`\n\n`x <- λ 2 1. f a 2`\n\n`x <- λ 2 1. f a a`","title":"Example: xaa = faa - Unification Examples","ref":"unification_examples.html#example-xaa-faa"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_ii)\nt_f = mk_const_term(\"f\", type_ii)\nt_a = mk_const_term(\"a\", type_i)\n\nxfa = mk_appl_term(t_x, mk_appl_term(t_f, t_a))\nfxa = mk_appl_term(t_f, mk_appl_term(t_x, t_a))\n\ndepth = 5\ninput = {xfa, fxa}\n\nresult = unify(input, true, depth)\npp_res(result)\n```\n\nThis finds four solutions:\n\n`x <- λ 1. 1 `\n\n`x <- λ 1. f 1`\n\n`x <- λ 1. f (f 1)`\n\n`x <- λ 1. f (f (f 1))`\n\nHowever the maximum search depth was also reached twice. This means that it might not have found all solutions. Try changing the `depth` value and see what other solutions can be found!","title":"Example: x(fa) = f(xa) - Unification Examples","ref":"unification_examples.html#example-x-fa-f-xa"},{"type":"extras","doc":"- xy+z=21\n- x+y+z=10\n- xz+y=9\n\n```elixir\nt_x = mk_free_var_term(\"x\", type_ii_ii)\nt_y = mk_free_var_term(\"y\", type_ii_ii)\nt_z = mk_free_var_term(\"z\", type_ii_ii)\n\ninput = [\n  {plus(mult(t_x, t_y), t_z), mk_num(21)},\n  {plus(plus(t_x, t_y), t_z), mk_num(10)},\n  {plus(mult(t_x, t_z), t_y), mk_num(9)}\n]\n\ndepth = 50\nresult = unify(input, true, depth)\npp_res(result, true)\n```\n\nTwo Solutions are found here:\n\nz = 1, y = 4, x = 5: `z <- λ 2 1. 2 1` | `y <- λ 2 1. 2 (2 (2 (2 1)))` | `x <- λ 2 1. 2 (2 (2 (2 (2 1))))`\n\nz = 1, y = 5, x = 4: `z <- λ 2 1. 2 1` | `y <- λ 2 1. 2 (2 (2 (2 (2 1))))` | `x <- λ 2 1. 2 (2 (2 (2 1)))`\n\nOnce again the maximum search depth was reached multiple times. However increasing the search depth will not yield more solutions here. Unification doesn't always terminate even when no more solutions can be found!","title":"Example: Multiple Equations - Unification Examples","ref":"unification_examples.html#example-multiple-equations"},{"type":"extras","doc":"```elixir\nt_x = mk_free_var_term(\"x\", type_i)\nt_z = mk_free_var_term(\"z\", type_ii)\nt_c = mk_const_term(\"c\", type_i)\nappl_zc = mk_appl_term(t_z, t_c)\n\ninput = {t_x, appl_zc}\nresult = unify(input)\npp_res(result)\n```\n\nHere the only given result is the tuple `{x,(z c)}` on the flexlist. The tuples on the flexlist are not resolved, since this algorithm only does pre-unification. Many (often infinite) substitutions can be created from these tuples. This functionality is not currently present in this module.","title":"Example: x = zc - Unification Examples","ref":"unification_examples.html#example-x-zc"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}